//////////////////////////////////////////////////////////////////////////
/// \file Interfaces.h
/// \brief Interfaces for the meta library
/// \author Anilkumar Patro
/// \date 5/26/2003
//////////////////////////////////////////////////////////////////////////

#ifndef __INTERFACES_H__
#define __INTERFACES_H__

#include <iosfwd>
#include <map>
#include <meta/Type.h>

struct ISerializable
{
	/// get the stream representation of this meta information
	virtual bool		unmarshal(std::istream &is, ISerializable *parent) =0;
	/// get the meta information from the stream representation
	virtual bool		marshal(std::ostream& s, ISerializable *parent) =0;
	
	virtual				~ISerializable() {}
};

struct IValue
{
  /// \name Type support
  //@{
	/// see if this type of this value is the same as the given type
	virtual bool		isOfType(Type type) const =0;

	/// get the type of this value
	virtual Type		getType() const =0;
  //@}

		
  /// \name String representation support
  //@{
	/// get the string representation of the value
	/// this is usually shorter representation of the value
	/// (for eg. string -> shorter version of the complete string, uri)
	virtual const char* getStringRepresentation() const =0;

	/// get the actual contents of the value
	///  eg. 1. uri -> will give the contents of the file.
	///      2. string -> will give the complete string
	/// \see getStringRepresentation()
	virtual const char* getStringContents() const =0;
  //@}

  /// \name Copy by value support
  //@{
	/// create a clone of this value
	virtual IValue*		clone() const =0;
  //@}

	virtual				~IValue() {}
};

struct IMappingIterator
{
	virtual bool hasNext() const =0;
	
	virtual void next(float *fValue, IValue **vValue) =0;

	virtual ~IMappingIterator() {}
};

struct IDimension : public ISerializable
{
	/// get name for this dimension
	virtual const char*	getName() const =0;
	/// set the name for this dimension
	virtual void		setName(const char *name) =0;

	/// get the type of values this dimension can hold
	virtual Type		getType() const =0;
	/// set the type of values this dimension can hold
	virtual void		setType(Type t) =0;

	virtual IValue*		getMappedValue(float f) const =0;
	virtual IMappingIterator *getMappingIterator() =0;

	virtual void		setMapping(float f, IValue *value) =0;
	
	/// compares two values of this dimension type to see if they are equal
	virtual bool		equal() =0;
	
	virtual				~IDimension() {}
};

struct IMeta : public ISerializable
{
	
	/// get the number of dimensions in the meta
	virtual int			getDimensionCount() const =0;
	/// set the number of dimensions in the meta
	virtual void		setDimensionCount(int dimensions) =0;
	
	/// get the number of datapoints in the meta
	virtual int			getDataPointCount() const =0;
	/// get the number of datapoints in the meta
	virtual void		setDataPointCount(int datapoints) =0;
	
	/// get a dimension given an index location
	virtual IDimension *getDimension(int dim) const =0;
	/// get a dimension given the string name
	virtual IDimension *getDimension(const char* dimName) const =0;

	virtual IDimension*	createDimension(const char* dimName, Type dimType) const =0;

	/// add a dimension to the meta information
	virtual void		addDimension(IDimension *dimension) =0;
	/// remove a dimension to the meta information
	virtual void		remDimension(IDimension *dimension) =0;
	
	virtual				~IMeta() {}
};

#endif //__INTERFACES_H__
